///////////////////////////////////////////////////////////////////////////////
// FileManager.h
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <vector>

#include "ExtendedFileWrapper.h"
#include "SectionHandler.h"
#include "FileContents.h"

class VFileManager
{
private:
	VExtendedFileWrapper m_FileWrapper;

	std::vector<VSectionHandler *> m_pSectionHandlers;

	void InitStandardHandlers();
	VSectionHandler * FindHandler(std::string a_strSectionName);

	bool ReadStaticDataSections(VFileContents & a_rContents);
	bool ReadDynamicData(VFileContents & a_rContents);
//	bool ReadNextSection(VFileContents & a_rContents);
	bool ReadNextDynamicRecord(VFileDynamicRecord * a_pDynRecord);
	bool ReadDynamicRecord(VFileContents & a_rContents);
	// The new and improved stuff

	bool ReadFileHeader(SSHORT & a_rVersionNum);
	bool ReadSectionHeader(SLONG & a_rSectionLen, std::string & a_rstrSectionName);
	bool ReadSectionBody(VFileContents & a_rContents, SLONG a_SectionLen, std::string a_strSectionName);
public:
	VFileManager();
	virtual ~VFileManager();

	bool CreateVFile(std::string a_strFilename);	// Create a new file for writing.. must use open after
	bool CreateVFile(const char * a_sxFilename);
		VExtendedFileWrapper::EFileMode OpenVFile(std::string a_strFilename);	// Open an existing file for read/edit
	void CloseVFile();

	bool WriteVFileHeader();

	void ReadSectionNames(std::vector<std::string> & a_rNames);
	bool ReadSection(std::string strName, VFileSection * a_pNewSection);

	bool WriteSection(VFileSection * a_pNewSection);
	bool WriteStaticDataTerminator();

	bool WriteDynamicRecord(VFileDynamicRecord * a_pDynRec);
	void WriteFlush();
		
	VFileContents * ReadVFile();					// This reads the entire file..
	void WriteVFile(VFileContents * a_pContents);	// ..and this writes it

	void RegisterHandler(VSectionHandler * a_pHandler);
	void RegisterDefaultHandler(VSectionHandler * a_pHandler);

#ifdef _DEBUG
	void Flush();
#endif
};

