#pragma once

#include <maya/MStatus.h>
#include <maya/MDagModifier.h>
#include <maya/MPxFileTranslator.h>
#include <maya/MObject.h>
#include <maya/MFnPlugin.h>
#include <maya/MStringArray.h>
#include <maya/MFnAnimCurve.h>
#include "FileManager.h"
#include "Body.h"

#include <vector>
#include <string>

#define VFILE_PLUGIN_NAME		"Vicon V-file"

class VFileTranslator : public MPxFileTranslator 
{
public:
	VFileTranslator();
	virtual ~VFileTranslator();

	static void * Creator();
	
	virtual bool haveReadMethod() const		{ return true; }
	virtual bool haveWriteMethod() const	{ return false; }
	virtual bool canBeOpened() const		{ return false; }

	virtual MString defaultExtension () const;

	virtual MString filter () const;

	MPxFileTranslator::MFileKind identifyFile (const MFileObject &, const char*, short) const;
	MStatus reader(const MFileObject & file, const MString & optionsString, MPxFileTranslator::FileAccessMode mode);

private:
	VFileManager m_FileManager;

	std::vector<VMayaBody *> m_Bodies;

	bool m_bRegisterNodeFail;

	void CleanUp();
	MObject GetAnimCurve(MPlug & a_rCurrentPlug);
	void ExtractBodyAndMarkerLabels(std::vector<std::string> & i_rDofLabels, std::vector<std::string> & o_rGlobalBodyLabels, std::vector<std::string> & o_rLocalBodyLabels, std::vector<std::string> & o_rMarkerLabels);
	void DeriveGroupAndLocatorName(std::string i_strGlobalName, std::string & o_strGroupName, std::string & o_strBodyName);
	MObject FindOrCreateDofLocator(std::string i_strGlobalName, std::string i_strGroupName, std::string i_strBodyName);
	MObject FindLocatorRecursive(std::string i_strLocatorName, std::string i_strParentName);
	bool IsLocatorName(std::string a_strObjectName);
	MObject CreateMarkerLocator(std::string i_strGlobalName, std::string i_strGroupName, std::string i_strBodyName);

	bool IsBody(std::string a_strObjName);
	bool IsGlobal(std::string a_strObjName);
	std::string StripSuffix(std::string a_strTarget);
	void SetupKeyframes(VFileContents * a_pContents, int a_CurDatagroup);
	void CreateOffsetCubes(std::vector<std::string> & a_rLocators);
};





